@@ -27,6 +27,8 @@ Copyright 2000-2004 Aurlien Gteau
 #include <kconfig.h>
 #include <kdebug.h>
 #include <kdeversion.h>
+#include <ksharedptr.h>
+#include <kstaticdeleter.h>
 #include <kio/global.h>
 
 #include "cache.moc"
@@ -48,17 +50,89 @@ namespace Gwenview {
 
 const char CONFIG_CACHE_MAXSIZE[]="maxSize";
 
+
+struct ImageData : public KShared {
+	ImageData( const KURL& url, const QDateTime& _timestamp )
+	: timestamp(_timestamp)
+	, age(0)
+	, fast_url( url.isLocalFile() && !KIO::probably_slow_mounted( url.path()))
+	, priority( false ) {
+	}
+
+	void addFile( const QByteArray& file );
+	void addImage( const ImageFrames& frames, const QCString& format );
+	void addThumbnail( const QPixmap& thumbnail, QSize imagesize );
+	long long cost() const;
+	int size() const;
+	QByteArray file;
+	ImageFrames frames;
+	QPixmap thumbnail;
+	QSize imagesize;
+	QCString format;
+	QDateTime timestamp;
+	mutable int age;
+	bool fast_url;
+	bool priority;
+	int fileSize() const;
+	int imageSize() const;
+	int thumbnailSize() const;
+	bool reduceSize();
+	bool isEmpty() const;
+
+	typedef KSharedPtr<ImageData> Ptr;
+};
+
+typedef QMap<KURL, ImageData::Ptr> ImageMap;
+
+struct Cache::Private {
+	ImageMap mImages;
+	int mMaxSize;
+	int mThumbnailSize;
+	QValueList< KURL > mPriorityURLs;
+
+
+	/**
+	 * This function tries to returns a valid ImageData for url and timestamp.
+	 * If it can't find one, it will create a new one and return it.
+	 */
+	ImageData::Ptr getOrCreateImageData(const KURL& url, const QDateTime& timestamp) {
+		if (mImages.contains(url)) {
+			ImageData::Ptr data = mImages[url];
+			if (data->timestamp == timestamp) return data;
+		}
+
+		ImageData::Ptr data = new ImageData(url, timestamp);
+		mImages[url] = data;
+		if (mPriorityURLs.contains(url)) data->priority = true;
+		return data;
+	}
+};
+
+
 Cache::Cache()
-: mMaxSize( DEFAULT_MAXSIZE )
-, mThumbnailSize( 0 ) // don't remember size for every thumbnail, but have one global and dump all if needed
-, mUsageRefcount( 0 )
 {
-    connect( &mCleanupTimer, SIGNAL( timeout()), SLOT( cleanupTimeout()));
+	d = new Private;
+	d->mMaxSize = DEFAULT_MAXSIZE;
+	// don't remember size for every thumbnail, but have one global and dump all if needed
+	d->mThumbnailSize = 0;
 }
 
+
+Cache::~Cache() {
+	d->mImages.clear();
+	delete d;
+}
+
+
+static Cache* sCache;
+static KStaticDeleter<Cache> sCacheDeleter;
+
+
 Cache* Cache::instance() {
-	static Cache manager;
-	return &manager;
+	if (!sCache) {
+		sCacheDeleter.setObject(sCache, new Cache());
+	}
+	return sCache;
 }
 
 // Priority URLs are used e.g. when prefetching for the slideshow - after an image is prefetched,
@@ -69,52 +143,31 @@ Cache* Cache::instance() {
 // will be even enlarged as necessary if needed.
 void Cache::setPriorityURL( const KURL& url, bool set ) {
 	if( set ) {
-		mPriorityURLs.append( url );
-		if( mImages.contains( url )) {
-			mImages[ url ].priority = true;
+		d->mPriorityURLs.append( url );
+		if( d->mImages.contains( url )) {
+			d->mImages[ url ]->priority = true;
 		}
 	} else {
-		mPriorityURLs.remove( url );
-		if( mImages.contains( url )) {
-			mImages[ url ].priority = false;
+		d->mPriorityURLs.remove( url );
+		if( d->mImages.contains( url )) {
+			d->mImages[ url ]->priority = false;
 		}
 		checkMaxSize();
 	}
 }
 
+
 void Cache::addFile( const KURL& url, const QByteArray& file, const QDateTime& timestamp ) {
 	LOG(url.prettyURL());
 	updateAge();
-	bool insert = true;
-	if( mImages.contains( url )) {
-		ImageData& data = mImages[ url ];
-		if( data.timestamp == timestamp ) {
-			data.addFile( file );
-			insert = false;
-		}
-	}
-	if( insert ) {
-		mImages[ url ] = ImageData( url, file, timestamp );
-		if( mPriorityURLs.contains( url )) mImages[ url ].priority = true;
-	}
+	d->getOrCreateImageData(url, timestamp)->addFile(file);
 	checkMaxSize();
 }
 
 void Cache::addImage( const KURL& url, const ImageFrames& frames, const QCString& format, const QDateTime& timestamp ) {
 	LOG(url.prettyURL());
 	updateAge();
-	bool insert = true;
-	if( mImages.contains( url )) {
-		ImageData& data = mImages[ url ];
-		if( data.timestamp == timestamp ) {
-			data.addImage( frames, format );
-			insert = false;
-		}
-	}
-	if( insert ) {
-		mImages[ url ] = ImageData( url, frames, format, timestamp );
-		if( mPriorityURLs.contains( url )) mImages[ url ].priority = true;
-	}
+	d->getOrCreateImageData(url, timestamp)->addImage(frames, format);
 	checkMaxSize();
 }
 
@@ -122,89 +175,80 @@ void Cache::addThumbnail( const KURL& url, const QPixmap& thumbnail, QSize image
 // Thumbnails are many and often - things would age too quickly. Therefore
 // when adding thumbnails updateAge() is called from the outside only once for all of them.
 //	updateAge();
-	bool insert = true;
-	if( mImages.contains( url )) {
-		ImageData& data = mImages[ url ];
-		if( data.timestamp == timestamp ) {
-			data.addThumbnail( thumbnail, imagesize );
-			insert = false;
-		}
-	}
-	if( insert ) {
-		mImages[ url ] = ImageData( url, thumbnail, imagesize, timestamp );
-		if( mPriorityURLs.contains( url )) mImages[ url ].priority = true;
-	}
+	d->getOrCreateImageData(url, timestamp)->addThumbnail(thumbnail, imagesize);
 	checkMaxSize();
 }
 
 void Cache::invalidate( const KURL& url ) {
-	mImages.remove( url );
+	d->mImages.remove( url );
 }
 
 QDateTime Cache::timestamp( const KURL& url ) const {
 	LOG(url.prettyURL());
-	if( mImages.contains( url )) return mImages[ url ].timestamp;
+	if( d->mImages.contains( url )) return d->mImages[ url ]->timestamp;
 	return QDateTime();
 }
 
 QByteArray Cache::file( const KURL& url ) const {
 	LOG(url.prettyURL());
-	if( mImages.contains( url )) {
-		const ImageData& data = mImages[ url ];
-		if( data.file.isNull()) return QByteArray();
-		data.age = 0;
-		return data.file;
+	if( d->mImages.contains( url )) {
+		const ImageData::Ptr data = d->mImages[ url ];
+		if( data->file.isNull()) return QByteArray();
+		data->age = 0;
+		return data->file;
 	}
 	return QByteArray();
 }
 
-void Cache::getFrames( const KURL& url, ImageFrames& frames, QCString& format ) const {
+void Cache::getFrames( const KURL& url, ImageFrames* frames, QCString* format ) const {
 	LOG(url.prettyURL());
-	frames=ImageFrames();
-	format=QCString();
-	if( mImages.contains( url )) {
-		const ImageData& data = mImages[ url ];
-		if( data.frames.isEmpty()) return;
-		frames = data.frames;
-		format = data.format;
-		data.age = 0;
+	Q_ASSERT(frames);
+	Q_ASSERT(format);
+	frames->clear();
+	*format = QCString();
+	if( d->mImages.contains( url )) {
+		const ImageData::Ptr data = d->mImages[ url ];
+		if( data->frames.isEmpty()) return;
+		*frames = data->frames;
+		*format = data->format;
+		data->age = 0;
 	}
 }
 
 QPixmap Cache::thumbnail( const KURL& url, QSize& imagesize ) const {
-	if( mImages.contains( url )) {
-		const ImageData& data = mImages[ url ];
-		if( data.thumbnail.isNull()) return QPixmap();
-		imagesize = data.imagesize;
+	if( d->mImages.contains( url )) {
+		const ImageData::Ptr data = d->mImages[ url ];
+		if( data->thumbnail.isNull()) return QPixmap();
+		imagesize = data->imagesize;
 //		data.age = 0;
-		return data.thumbnail;
+		return data->thumbnail;
 	}
 	return QPixmap();
 }
 
 void Cache::updateAge() {
-	for( QMap< KURL, ImageData >::Iterator it = mImages.begin();
-		it != mImages.end();
+	for( ImageMap::Iterator it = d->mImages.begin();
+		it != d->mImages.end();
 		++it ) {
-		(*it).age++;
+		(*it)->age++;
 	}
 }
 
 void Cache::checkThumbnailSize( int size ) {
-	if( size != mThumbnailSize ) {
+	if( size != d->mThumbnailSize ) {
 		// simply remove all thumbnails, should happen rarely
-		for( QMap< KURL, ImageData >::Iterator it = mImages.begin();
-			it != mImages.end();
+		for( ImageMap::Iterator it = d->mImages.begin();
+			it != d->mImages.end();
 			) {
-			if( !(*it).thumbnail.isNull()) {
-				QMap< KURL, ImageData >::Iterator it2 = it;
+			if( !(*it)->thumbnail.isNull()) {
+				ImageMap::Iterator it2 = it;
 				++it;
-				mImages.remove( it2 );
+				d->mImages.remove( it2 );
 			} else {
 				++it;
 			}
 		}
-		mThumbnailSize = size;
+		d->mThumbnailSize = size;
 	}
 }
 
@@ -215,32 +259,32 @@ static KURL _cache_url; // hack only for debugging for item to show also its key
 void Cache::checkMaxSize() {
 	for(;;) {
 		int size = 0;
-		QMap< KURL, ImageData >::Iterator max;
+		ImageMap::Iterator max;
 		long long max_cost = -1;
 #ifdef DEBUG_CACHE
 		int with_file = 0;
 		int with_thumb = 0;
 		int with_image = 0;
 #endif
-		for( QMap< KURL, ImageData >::Iterator it = mImages.begin();
-			it != mImages.end();
+		for( ImageMap::Iterator it = d->mImages.begin();
+			it != d->mImages.end();
 			++it ) {
-			size += (*it).size();
+			size += (*it)->size();
 #ifdef DEBUG_CACHE
 			if( !(*it).file.isNull()) ++with_file;
 			if( !(*it).thumbnail.isNull()) ++with_thumb;
 			if( !(*it).frames.isEmpty()) ++with_image;
 #endif
-			long long cost = (*it).cost();
-			if( cost > max_cost && ! (*it).priority ) {
+			long long cost = (*it)->cost();
+			if( cost > max_cost && ! (*it)->priority ) {
 				max_cost = cost;
 				max = it;
 			}
 		}
-		if( size <= mMaxSize || max_cost == -1 ) {
+		if( size <= d->mMaxSize || max_cost == -1 ) {
 #if 0
 #ifdef DEBUG_CACHE
-			kdDebug() << "Cache: Statistics (" << mImages.size() << "/" << with_file << "/"
+			kdDebug() << "Cache: Statistics (" << d->mImages.size() << "/" << with_file << "/"
 				<< with_thumb << "/" << with_image << ")" << endl;
 #endif
 #endif
@@ -250,90 +294,48 @@ void Cache::checkMaxSize() {
 		_cache_url = max.key();
 #endif
 
-		if( !(*max).reduceSize() || (*max).isEmpty()) mImages.remove( max );
+		if( !(*max)->reduceSize() || (*max)->isEmpty()) d->mImages.remove( max );
 	}
 }
 
 void Cache::readConfig(KConfig* config,const QString& group) {
 	KConfigGroupSaver saver( config, group );
-	mMaxSize = config->readNumEntry( CONFIG_CACHE_MAXSIZE, mMaxSize );
+	d->mMaxSize = config->readNumEntry( CONFIG_CACHE_MAXSIZE, d->mMaxSize );
 	checkMaxSize();
 }
 
-void Cache::ref() {
-	++mUsageRefcount;
-	mCleanupTimer.stop();
-}
 
-void Cache::deref() {
-	if( --mUsageRefcount == 0 ) mCleanupTimer.start( 60000, true );
-}
-
-void Cache::cleanupTimeout() {
-	mImages.clear();
-}
-
-Cache::ImageData::ImageData( const KURL& url, const QByteArray& f, const QDateTime& t )
-: file( f )
-, timestamp( t )
-, age( 0 )
-, fast_url( url.isLocalFile() && !KIO::probably_slow_mounted( url.path()))
-, priority( false )
-{
-	file.detach(); // explicit sharing
-}
-
-Cache::ImageData::ImageData( const KURL& url, const ImageFrames& frms, const QCString& f, const QDateTime& t )
-: frames( frms )
-, format( f )
-, timestamp( t )
-, age( 0 )
-, fast_url( url.isLocalFile() && !KIO::probably_slow_mounted( url.path()))
-, priority( false )
-{
-}
-
-Cache::ImageData::ImageData( const KURL& url, const QPixmap& thumb, QSize imgsize, const QDateTime& t )
-: thumbnail( thumb )
-, imagesize( imgsize )
-, timestamp( t )
-, age( 0 )
-, fast_url( url.isLocalFile() && !KIO::probably_slow_mounted( url.path()))
-, priority( false )
-{
-}
-
-void Cache::ImageData::addFile( const QByteArray& f ) {
+void ImageData::addFile( const QByteArray& f ) {
 	file = f;
 	file.detach(); // explicit sharing
 	age = 0;
 }
 
-void Cache::ImageData::addImage( const ImageFrames& fs, const QCString& f ) {
+void ImageData::addImage( const ImageFrames& fs, const QCString& f ) {
 	frames = fs;
 	format = f;
 	age = 0;
 }
 
-void Cache::ImageData::addThumbnail( const QPixmap& thumb, QSize imgsize ) {
+void ImageData::addThumbnail( const QPixmap& thumb, QSize imgsize ) {
 	thumbnail = thumb;
 	imagesize = imgsize;
 //	age = 0;
 }
 
-int Cache::ImageData::size() const {
+int ImageData::size() const {
 	return QMAX( fileSize() + imageSize() + thumbnailSize(), 100 ); // some minimal size per item
 }
 
-int Cache::ImageData::fileSize() const {
+int ImageData::fileSize() const {
 	return !file.isNull() ? file.size() : 0;
 }
 
-int Cache::ImageData::thumbnailSize() const {
+int ImageData::thumbnailSize() const {
 	return !thumbnail.isNull() ? thumbnail.height() * thumbnail.width() * thumbnail.depth() / 8 : 0;
 }
 
-int Cache::ImageData::imageSize() const {
+int ImageData::imageSize() const {
 	int ret = 0;
 	for( ImageFrames::ConstIterator it = frames.begin(); it != frames.end(); ++it ) {
 		ret += (*it).image.height() * (*it).image.width() * (*it).image.depth() / 8;
@@ -341,7 +343,7 @@ int Cache::ImageData::imageSize() const {
 	return ret;
 }
 
-bool Cache::ImageData::reduceSize() {
+bool ImageData::reduceSize() {
 	if( !file.isNull() && fast_url && !frames.isEmpty()) {
 		file = QByteArray();
 #ifdef DEBUG_CACHE
@@ -378,11 +380,11 @@ bool Cache::ImageData::reduceSize() {
 	return false; // reducing here would mean clearing everything
 }
 
-bool Cache::ImageData::isEmpty() const {
+bool ImageData::isEmpty() const {
 	return file.isNull() && frames.isEmpty() && thumbnail.isNull();
 }
 
-long long Cache::ImageData::cost() const {
+long long ImageData::cost() const {
 	long long s = size();
 	if( fast_url && !file.isNull()) {
 		s *= ( format == "JPEG" ? 10 : 100 ); // heavy penalty for storing local files
