@@ -42,58 +42,25 @@ class LIBGWENVIEW_EXPORT Cache : public QObject {
 Q_OBJECT
 public:
 	static Cache* instance();
+	~Cache();
 	void addImage( const KURL& url, const ImageFrames& frames, const QCString& format, const QDateTime& timestamp );
 	void addFile( const KURL& url, const QByteArray& file, const QDateTime& timestamp );
 	void addThumbnail( const KURL& url, const QPixmap& thumbnail, QSize imagesize, const QDateTime& timestamp );
 	QDateTime timestamp( const KURL& url ) const;
 	QByteArray file( const KURL& url ) const;
-	void getFrames( const KURL& url, ImageFrames& frames, QCString& format ) const;
+	void getFrames( const KURL& url, ImageFrames* frames, QCString* format ) const;
 	QPixmap thumbnail( const KURL& url, QSize& imagesize ) const;
 	void setPriorityURL( const KURL& url, bool set );
 	void invalidate( const KURL& url );
 	void checkThumbnailSize( int size );
 	void readConfig(KConfig*,const QString& group);
 	void updateAge();
-	void ref();
-	void deref();
 	enum { DEFAULT_MAXSIZE = 16 * 1024 * 1024 }; // 16MiB
 private:
 	Cache();
 	void checkMaxSize();
-	struct ImageData {
-		ImageData( const KURL& url, const QByteArray& file, const QDateTime& timestamp );
-		ImageData( const KURL& url, const QImage& image, const QCString& format, const QDateTime& timestamp );
-		ImageData( const KURL& url, const ImageFrames& frames, const QCString& format, const QDateTime& timestamp );
-		ImageData( const KURL& url, const QPixmap& thumbnail, QSize imagesize, const QDateTime& timestamp );
-		void addFile( const QByteArray& file );
-		void addImage( const ImageFrames& frames, const QCString& format );
-		void addThumbnail( const QPixmap& thumbnail, QSize imagesize );
-		long long cost() const;
-		int size() const;
-		QByteArray file;
-		ImageFrames frames;
-		QPixmap thumbnail;
-		QSize imagesize;
-		QCString format;
-		QDateTime timestamp;
-		mutable int age;
-		bool fast_url;
-		bool priority;
-		int fileSize() const;
-		int imageSize() const;
-		int thumbnailSize() const;
-		bool reduceSize();
-		bool isEmpty() const;
-		ImageData() {}; // stupid QMap
-	};
-	QMap< KURL, ImageData > mImages;
-	int mMaxSize;
-	int mThumbnailSize;
-	int mUsageRefcount;
-	QTimer mCleanupTimer;
-	QValueList< KURL > mPriorityURLs;
-private slots:
-	void cleanupTimeout();
+	class Private;
+	Private* d;
 };
 
 } // namespace
