@@ -29,10 +29,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 // KDE
 #include <kapplication.h>
+#include <kimageio.h>
 #include <kmimetype.h>
 
 // Local
 #include "cache.h"
+#include "miscconfig.h"
 #include "imageutils/imageutils.h"
 #include "imageutils/jpegcontent.h"
 
@@ -436,6 +438,16 @@ void ImageLoader::slotGetResult(KIO::Job* job) {
 	}
 }
 
+// There is no way in KImageIO to get the mimeType from the image format.
+// This function assumes KImageIO::types and KImageIO::mimeTypes return items
+// in the same order (which they do, according to the source code).
+static QString mimeTypeFromFormat(const char* format) {
+	QStringList formats = KImageIO::types(KImageIO::Reading);
+	QStringList mimeTypes = KImageIO::mimeTypes(KImageIO::Reading);
+	int pos = formats.findIndex(QString::fromAscii(format));
+	Q_ASSERT(pos != -1);
+	return mimeTypes[pos];
+}
 
 void ImageLoader::slotDataReceived(KIO::Job* job, const QByteArray& chunk) {
 	LOG2("size: " << chunk.size());
@@ -447,9 +459,18 @@ void ImageLoader::slotDataReceived(KIO::Job* job, const QByteArray& chunk) {
 
 	if (oldSize==0) {
 		// Try to determine the data type
-		KMimeType::Ptr ptr = KMimeType::findByContent(d->mRawData);
-		d->mMimeType = ptr->name();
-		d->mURLKind = MimeTypeUtils::mimeTypeKind(d->mMimeType);
+		QBuffer buffer(d->mRawData);
+		buffer.open(IO_ReadOnly);
+		const char* format = QImageIO::imageFormat(&buffer);
+		if (format) {
+			// This is a raster image, get the mime type now
+			d->mURLKind = MimeTypeUtils::KIND_RASTER_IMAGE;
+			d->mMimeType = mimeTypeFromFormat(format);
+		} else {
+			KMimeType::Ptr ptr = KMimeType::findByContent(d->mRawData);
+			d->mMimeType = ptr->name();
+			d->mURLKind = MimeTypeUtils::mimeTypeKind(d->mMimeType);
+		}
 		if (d->mURLKind!=MimeTypeUtils::KIND_RASTER_IMAGE) {
 			Q_ASSERT(!d->mDecoderTimer.isActive());
 			job->kill(true /* quietly */);
@@ -662,15 +683,18 @@ void ImageLoader::changed(const QRect& constRect) {
 		Q_ASSERT(!d->mImageFormat.isEmpty());
 		if (d->mImageFormat == "JPEG") {
 			// This is a JPEG, extract orientation and adjust mProcessedImage
-			// if necessary
+			// if necessary according to misc options
 			ImageUtils::JPEGContent content;
 
 			if (content.loadFromData(d->mRawData)) {
-				d->mOrientation = content.orientation();
-				if (d->mOrientation != ImageUtils::NOT_AVAILABLE && d->mOrientation != ImageUtils::NORMAL) {
+				d->mOrientation = content.orientation(); 
+				if (MiscConfig::autoRotateImages() && 
+					d->mOrientation != ImageUtils::NOT_AVAILABLE && d->mOrientation != ImageUtils::NORMAL) {
 					QSize size = content.size();
 					d->mProcessedImage = QImage(size, d->mDecoder.image().depth());
 				}
+				d->mProcessedImage.setDotsPerMeterX(content.dotsPerMeterX());
+				d->mProcessedImage.setDotsPerMeterY(content.dotsPerMeterY());
 			} else {
 				kdWarning() << "ImageLoader::changed(): JPEGContent could not load '" << d->mURL.prettyURL() << "'\n";
 			}
@@ -680,8 +704,9 @@ void ImageLoader::changed(const QRect& constRect) {
 		emit sizeLoaded(d->mProcessedImage.width(), d->mProcessedImage.height());
 	}
 
-	// Apply orientation if necessary
-	if (d->mOrientation != ImageUtils::NOT_AVAILABLE && d->mOrientation != ImageUtils::NORMAL) {
+	// Apply orientation if necessary and if wanted by user settings (misc options)
+	if (MiscConfig::autoRotateImages() && 
+		d->mOrientation != ImageUtils::NOT_AVAILABLE && d->mOrientation != ImageUtils::NORMAL) {
 		// We can only rotate whole images, so copy the loaded rect in a temp
 		// image, rotate the temp image and copy it to mProcessedImage
 
