@@ -21,15 +21,13 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #ifndef MIMETYPEUTILS_H
 #define MIMETYPEUTILS_H   
 
-// Qt
-#include <qcstring.h>
-
 // Local
 #include "libgwenview_export.h"
 
 class KFileItem;
 class KURL;
 
+class QString;
 class QStringList;
 
 namespace Gwenview {
@@ -40,7 +38,7 @@ enum Kind { KIND_UNKNOWN, KIND_DIR, KIND_ARCHIVE, KIND_FILE, KIND_RASTER_IMAGE }
 LIBGWENVIEW_EXPORT const QStringList& rasterImageMimeTypes();
 Kind fileItemKind(const KFileItem*);
 Kind urlKind(const KURL&);
-Kind determineKindFromContent(const QByteArray&);
+Kind mimeTypeKind(const QString& mimeType);
 
 } // namespace FileUtils
 
