@@ -43,12 +43,22 @@ public:
 	
 	Orientation orientation() const;
 	void resetOrientation();
+  
+	int dotsPerMeterX() const;
+	int dotsPerMeterY() const;
 
 	QSize size() const;
 
 	QString comment() const;
 	void setComment(const QString&);
-	
+
+	QString aperture() const;
+	QString exposureTime() const;
+	QString iso() const;
+	QString focalLength() const;
+
+	QString getExifInformation(const QString exifkey) const;
+
 	void transform(Orientation);
 
 	QImage thumbnail() const;
@@ -66,6 +76,7 @@ private:
 	JPEGContent(const JPEGContent&);
 	void operator=(const JPEGContent&);
     void applyPendingTransformation();
+	int dotsPerMeter(const QString& keyName) const;
 };
 
 
